/*
 *      @(#)Copyright (c) 1993, Simon J. Gerraty.
 *      
 *      This code is presented AS IS.  It comes with NO WARRANTY.
 *      Permission to use, modify and distribute this source code 
 *      is granted subject to the following conditions.
 *      1/ that the above copyright notice and this notice 
 *      are preserved in all copies and that due credit be given 
 *      to the author.  
 *      2/ that any changes to this code are clearly commented 
 *      as such so that the author does not get blamed for bugs 
 *      other than his own.
 *      
 *      Please send copies of changes and bug-fixes to:
 *      sjg@quick.com.au
 */

#ifndef lint
static char RCSid[] = "$Id: ipcallid.c,v 1.7 1998/11/13 02:41:12 sjg Exp $";
#endif
#include <stdio.h>
#ifdef __STDC__
#include <stdlib.h>
#endif
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <string.h>
#include <fcntl.h>
#include <ctype.h>
#include <errno.h>
#include <signal.h>
#include <syslog.h>

#ifndef MAXNAMLEN
# define MAXNAMLEN 256
#endif
#ifdef SOCKS
# include "socks.h"
#endif

#include "sjg/macros.h"

extern int errno;
extern char *Hostname();

char *
ipCallerId(sa, flags, faked)
	struct sockaddr_in *sa;
	int	flags;
	char **faked;
{
	static char	*hostname, caller_id[2 * MAXNAMLEN];
	static char hostbuf[MAXNAMLEN];
	static struct sockaddr_in last_sa;
	static int last_flags;
	char *hostaddr;
        struct hostent	*hp = NULL;
	struct servent	*sp = NULL;
	struct in_addr	in;
	register char	*rcp;
	int		port = ntohs(sa->sin_port);

	/*
	 * try and save some overhead
	 */
	if (last_flags == flags
	    && memcmp((char *) &last_sa, (char *)sa, sizeof(last_sa)) == 0) {
		if ((flags & 8) != 0
		    && faked != 0
		    && hostname != hostbuf)
			*faked = hostbuf;
		return caller_id;
	}
	last_flags = flags;
	memcpy((char *) &last_sa, (char *)sa, sizeof(last_sa));
	
	hostname = hostaddr = inet_ntoa(sa->sin_addr);

	if ((flags & 8) != 0
	    /*
	     * This lot ripped out of ssld.c (used by SSLrshd)
	     * so we can use it for other things
	     */
	    && strcmp(hostaddr, "127.0.0.1") == 0) {
		hostname = Hostname(1);
		hostname = ARRAY_STR(hostbuf, hostname);
	} else if (flags &&
	    (hp = gethostbyaddr((char *) &sa->sin_addr,
				sizeof(sa->sin_addr), AF_INET)) != NULL) {
		hostname = ARRAY_STR(hostbuf, hp->h_name);
		
		if (flags & 8) {
			/*
			 * ok we are being paranoid...
			 * make sure that gethostby{addr,name} agree
			 * not only on the address list, but the fully
			 * qualified name.
			 */
			hostname = "unknown";
			if ((hp = gethostbyname(hostbuf)) != 0
			    && strcmp(hostbuf, hp->h_name) == 0) {
			
				for (; ; hp->h_addr_list++) {
					if (hp->h_addr_list[0] == NULL) {
						syslog(LOG_NOTICE|LOG_AUTH,
						       "Address %s, not listed for host %s",
						       hostaddr,
						       hp->h_name);
						break;
					}
					if (memcmp(hp->h_addr_list[0],
						   (char *) &sa->sin_addr,
						   sizeof(sa->sin_addr)) == 0) {
						
						hostname = hostbuf;
						break;
					}
				}
			} else if (hp != 0) {
				/*
				 * Tell them - incase they are not using
				 * tcpd or libwrap.
				 */
				syslog(LOG_NOTICE|LOG_AUTH,
				       "warning: hostbyaddr(%s) != hostbyname(%s) = %s",
				       hostaddr, hostbuf, hp->h_name);
			}
			if (faked != 0
			    && hostname != hostbuf)
				*faked = hostbuf;
		}
	}
	if ((flags & ~16) < 2) 			/* keep it compatable */
		(void) sprintf(caller_id, "%s", hostname);
	else
		(void) sprintf(caller_id, "%s/%s", hostname, hostaddr);
	if (flags & 16) {
		rcp = strchr(caller_id, '\0');
		if (flags && (port == 0 || 
			      (sp = getservbyport((int) sa->sin_port, (char *) NULL))))
			(void) sprintf(rcp, ".%s", sp ? sp->s_name : "*");
		else
			(void) sprintf(rcp, ".%d", port);
	}
	return caller_id;
}

char *
ip_caller_id(sa, flags)
	struct sockaddr_in *sa;
	int	flags;
{
	return ipCallerId(sa, flags|16, 0);
}

char *
sock_id(s, flags)
	int s;
	int flags;
{
	struct sockaddr_in sa;
	int i = -1;
	int x;

	x = sizeof (sa);
	if (flags & 2) {			/* peer */
		i = getpeername(s, (struct sockaddr *)&sa, &x);
	} else {
		i = getsockname(s, (struct sockaddr *)&sa, &x);
	}
	if (i >= 0)
		return ipCallerId(&sa, (flags & 9)|16, 0);
	return (char *) 0;
}

